@@ -12,25 +12,32 @@ from kabaret.subprocess_manager.flow import RunAction
 from libreflow import baseflow
 
 from libreflow.utils.b3d import wrap_python_expr
+from libreflow.utils.flow import get_context_value
+
+from .file import FileSystemMap
+
 
 class Department(baseflow.departments.Department):
     _short_name = flow.Param(None)
     _file_prefix = flow.Computed(cached=True)
     _shot = flow.Parent(2)
     
+    file_prefix = flow.Param('{dept}').ui(
+        hidden=True,
+        editable=False,
+    )
+    file_path = flow.Param('{department}').ui(
+        hidden=True,
+        editable=False,
+    )
+    
     def compute_child_value(self, child_value):
         if child_value is self.path:
-            settings = get_contextual_dict(self, "settings")
-            path = os.path.join(
-                settings["film"],
-                settings["sequence"],
-                settings["shot"],
-                settings["department"],
-            )
-            child_value.set(path)
+            path = get_context_value(self, 'file_path', delim='/')
+            self.path.set(path)
         elif child_value is self._file_prefix:
-            settings = get_contextual_dict(self, "settings")
-            child_value.set("{film}_{sequence}_{shot}_{dept}_".format(**settings))
+            prefix = get_context_value(self, 'file_prefix', delim='_')
+            self._file_prefix.set(prefix + '_')
     
     def get_default_contextual_edits(self, context_name):
         if context_name == "settings":
@@ -674,6 +681,14 @@ class Shot(baseflow.film.Shot):
 
     _film = flow.Parent(4)
     departments = flow.Child(ShotDepartments).ui(expanded=True)
+    file_prefix = flow.Param('{shot}').ui(
+        hidden=True,
+        editable=False,
+    )
+    file_path = flow.Param('{shot}').ui(
+        hidden=True,
+        editable=False,
+    )
 
     def compute_child_value(self, child_value):
         if child_value is self.kitsu_url:
@@ -699,6 +714,15 @@ class Sequence(baseflow.film.Sequence):
         expanded=True,
         show_filter=True,
     )
+    file_prefix = flow.Param('{sequence}').ui(
+        hidden=True,
+        editable=False,
+    )
+    file_path = flow.Param('{sequence}').ui(
+        hidden=True,
+        editable=False,
+    )
+    misc = flow.Child(MiscDepartment)
 
     def compute_child_value(self, child_value):
         if child_value is self.kitsu_url:
@@ -735,6 +759,14 @@ class Film(flow.Object):
         expanded=True,
         show_filter=True,
     )
+    file_prefix = flow.Param('{film}').ui(
+        hidden=True,
+        editable=False,
+    )
+    file_path = flow.Param('{film}').ui(
+        hidden=True,
+        editable=False,
+    )
     
     def get_default_contextual_edits(self, context_name):
         if context_name == "settings":
