@@ -6,27 +6,29 @@ from kabaret.flow_contextual_dict import ContextualView, get_contextual_dict
 from kabaret.subprocess_manager.flow import RunAction
 
 from libreflow import baseflow
+from libreflow.utils.flow import get_context_value
 from .file import FileSystemMap
 
 
 class Department(baseflow.departments.Department):
     _short_name = flow.Param(None)
     _file_prefix = flow.Computed(cached=True)
+    file_prefix = flow.Param('{dept}').ui(
+        hidden=True,
+        editable=False,
+    )
+    file_path = flow.Param('{department}').ui(
+        hidden=True,
+        editable=False,
+    )
     
     def compute_child_value(self, child_value):
         if child_value is self.path:
-            settings = get_contextual_dict(self, "settings")
-            path = os.path.join(
-                "lib",
-                settings["asset_type"],
-                settings["asset_family"],
-                settings["asset_name"],
-                settings["department"],
-            )
-            child_value.set(path)
+            path = get_context_value(self, 'file_path', delim='/')
+            self.path.set(path)
         elif child_value is self._file_prefix:
-            settings = get_contextual_dict(self, "settings")
-            child_value.set("lib_{asset_type}_{asset_family}_{asset_name}_{dept}_".format(**settings))
+            prefix = get_context_value(self, 'file_prefix', delim='_')
+            self._file_prefix.set(prefix + '_')
     
     def get_default_contextual_edits(self, context_name):
         if context_name == "settings":
@@ -52,44 +54,42 @@ class RiggingDepartment(Department):
 class Util(flow.Object):
 
     _util_map = flow.Parent()
-    path = flow.Computed().ui(hidden=True)
-    _file_prefix = flow.Computed()
     files = flow.Child(FileSystemMap)
+    file_prefix = flow.Param('{util}').ui(
+        hidden=True,
+        editable=False,
+    )
+    file_path = flow.Param('{util}').ui(
+        hidden=True,
+        editable=False,
+    )
 
-    def compute_child_value(self, child_value):
-        if child_value is self.path:
-            settings = get_contextual_dict(self, "settings")
-            path = os.path.join(
-                self._util_map.path.get(),
-                self.name(),
-            )
-            self.path.set(path)
-        elif child_value is self._file_prefix:
-            self._file_prefix.set(self._util_map._file_prefix.get() + self.name() + '_')
+    def get_default_contextual_edits(self, context_name):
+        if context_name == 'settings':
+            return dict(util=self.name())
 
 
 class UtilMap(flow.Map):
 
     _short_name = flow.Param(None)
-    _utils = flow.Parent()
-    _file_prefix = flow.Computed()
-    path = flow.Computed().ui(hidden=True)
     create = flow.Child(baseflow.maputils.SimpleCreateAction)
+    file_prefix = flow.Param('{util_type}').ui(
+        hidden=True,
+        editable=False,
+    )
+    file_path = flow.Param('{util_type}').ui(
+        hidden=True,
+        editable=False,
+    )
     
     @classmethod
     def mapped_type(cls):
         return Util
-    
-    def compute_child_value(self, child_value):
-        if child_value is self.path:
-            settings = get_contextual_dict(self, "settings")
-            path = os.path.join(
-                self._utils.path.get(),
-                self._short_name.get(),
-            )
-            self.path.set(path)
-        elif child_value is self._file_prefix:
-            self._file_prefix.set(self._utils._file_prefix.get() + self._short_name.get() + '_')
+
+    def get_default_contextual_edits(self, context_name):
+        if context_name == 'settings':
+            name = self._short_name.get() or self.name()
+            return dict(util_type=name)
 
 
 class SkinningUtils(UtilMap):
@@ -104,23 +104,22 @@ class RigModuleUtils(UtilMap):
 
 class Utils(flow.Object):
     
-    skinning = flow.Child(SkinningUtils)
-    rig_modules = flow.Child(RigModuleUtils)
-    path = flow.Computed().ui(hidden=True)
-    _file_prefix = flow.Computed()
-
-    def compute_child_value(self, child_value):
-        if child_value is self.path:
-            settings = get_contextual_dict(self, "settings")
-            path = os.path.join(
-                "lib",
-                settings["asset_type"],
-                "utils"
-            )
-            self.path.set(path)
-        elif child_value is self._file_prefix:
-            settings = get_contextual_dict(self, "settings")
-            child_value.set("lib_{asset_type}_utils_".format(**settings))
+    skinning = flow.Child(SkinningUtils).ui(
+        expanded=True,
+        show_filter=True,
+    )
+    rig_modules = flow.Child(RigModuleUtils).ui(
+        expanded=True,
+        show_filter=True,
+    )
+    file_prefix = flow.Param('utils').ui(
+        hidden=True,
+        editable=False,
+    )
+    file_path = flow.Param('utils').ui(
+        hidden=True,
+        editable=False,
+    )
 
 
 class AssetDepartments(flow.Object):
@@ -417,6 +416,14 @@ class Asset(baseflow.lib.Asset):
     _asset_type = flow.Parent(4)
     departments = flow.Child(AssetDepartments).ui(expanded=True)
     configure_action = flow.Child(ConfigureAssetAction)
+    file_prefix = flow.Param('{asset_name}').ui(
+        hidden=True,
+        editable=False,
+    )
+    file_path = flow.Param('{asset_name}').ui(
+        hidden=True,
+        editable=False,
+    )
 
     def compute_child_value(self, child_value):
         if child_value is self.kitsu_url:
@@ -440,6 +447,14 @@ class AssetFamily(flow.Object):
         expanded=True,
         show_filter=True,
     )
+    file_prefix = flow.Param('{asset_family}').ui(
+        hidden=True,
+        editable=False,
+    )
+    file_path = flow.Param('{asset_family}').ui(
+        hidden=True,
+        editable=False,
+    )
 
     def get_default_contextual_edits(self, context_name):
         if context_name == "settings":
@@ -459,6 +474,14 @@ class AssetType(flow.Object):
         show_filter=True,
     )
     utils = flow.Child(Utils).ui(expanded=False)
+    file_prefix = flow.Param('{asset_type}').ui(
+        hidden=True,
+        editable=False,
+    )
+    file_path = flow.Param('{asset_type}').ui(
+        hidden=True,
+        editable=False,
+    )
 
     def get_default_contextual_edits(self, context_name):
         if context_name == "settings":
@@ -604,6 +627,14 @@ class AssetLib(flow.Object):
     add_asset = flow.Child(CreateAssetAction).ui(
         label="Add asset"
     )
+    file_prefix = flow.Param('lib').ui(
+        hidden=True,
+        editable=False,
+    )
+    file_path = flow.Param('lib').ui(
+        hidden=True,
+        editable=False,
+    )
 
     def get_default_contextual_edits(self, context_name):
         if context_name == "settings":
